@@ -2,49 +2,57 @@ package Convert::Pluggable;
 
 use Moo;
 
-use File::Basename;
+use File::Slurp qw/read_file/;
 
 use Data::Float qw/float_is_infinite float_is_nan/;
-use Exporter qw(import);
-use JSON;
 use Scalar::Util qw/looks_like_number/;
 
+use JSON;
+use Switch;
+
 our @EXPORT_OK = qw(convert get_units);
 
-our $VERSION = '0.031';
+our $VERSION           = '0.032';
+our $DEFAULT_PRECISION = 3;
 
 has data_file => ( is => 'lazy', );
 
 has types => ( is => 'lazy', );
 
+sub temperature_get_factor {
+    my $convert = shift;
+
+    switch ( $convert->{'from_unit'} ) {
+        case /^(fahrenheit|f)$/i { return $convert->{'factor'}; }
+        case /^(celsius|c)$/i {
+            return ( $convert->{'factor'} * ( 9 / 5 ) ) + 32;
+        }
+        case /^(kelvin|k)$/i {
+            return ( $convert->{'factor'} * ( 9 / 5 ) ) - 459.67;
+        }
+        case /^(rankine|r)$/i { return ( $convert->{'factor'} - 491.67 ) + 32; }
+        case /^(reaumur|re)$/i {
+            return ( $convert->{'factor'} * ( 9 / 4 ) ) + 32;
+        }
+        else { return undef; }
+    }
+}
+
 sub convert_temperatures {
-    my $from   = shift;
-    my $to     = shift;
-    my $factor = shift;
+    my $convert = shift;
 
     # convert $from to fahrenheit:
-    if ( $from =~ /^(fahrenheit|f)$/i ) { $factor = $factor; }
-    elsif ( $from =~ /^(celsius|c)$/i ) {
-        $factor = ( $factor * ( 9 / 5 ) ) + 32;
-    }
-    elsif ( $from =~ /^(kelvin|k)$/i ) {
-        $factor = ( 9 / 5 ) * ( $factor - 273.15 ) + 32;
-    }
-    elsif ( $from =~ /^(rankine|r)$/i ) { $factor = $factor - 459.67; }
-    else { $factor = ( $factor * ( 9 / 4 ) ) + 32; }
+    my $factor = temperature_get_factor($convert);
 
     # convert fahrenheit $to:
-    if ( $to =~ /^(fahrenheit|f)$/i ) { $factor = $factor; }
-    elsif ( $to =~ /^(celsius|c)$/i ) {
-        $factor = ( $factor - 32 ) * ( 5 / 9 );
+    switch ( $convert->{'to_unit'} ) {
+        case /^(fahrenheit|f)$/i { return $factor; }
+        case /^(celsius|c)$/i    { return ( $factor - 32 ) * ( 5 / 9 ); }
+        case /^(kelvin|k)$/i     { return ( $factor + 459.67 ) * ( 5 / 9 ); }
+        case /^(rankine|r)$/i    { return $factor + 459.67; }
+        case /^(reaumur|re)$/i   { return ( $factor - 32 ) * ( 4 / 9 ); }
+        else                     { return undef; }
     }
-    elsif ( $to =~ /^(kelvin|k)$/i ) {
-        $factor = ( $factor + 459.67 ) * ( 5 / 9 );
-    }
-    elsif ( $to =~ /^(rankine|r)$/i ) { $factor = $factor + 459.67; }
-    else { $factor = ( $factor - 32 ) * ( 4 / 9 ); }
-
-    return $factor;
 }
 
 sub get_matches {
@@ -63,7 +71,7 @@ sub get_matches {
     my @units           = ();
     my @can_be_negative = ();
 
-    my @types = @{ get_units() } if !$self->types;
+    my @types = @{ get_units() };
 
     foreach my $match (@matches) {
         foreach my $type ( @{ $self->types } ) {
@@ -103,8 +111,9 @@ sub convert {
     my $conversion = shift;
     my $matches    = get_matches( $self,
         [ $conversion->{'from_unit'}, $conversion->{'to_unit'} ] );
-    
-    return if !( $matches->{'type_1'} && $matches->{'type_2'} );
+
+    return
+      if !( $matches->{'type_1'} && $matches->{'type_2'} );
 
     if ( looks_like_number( $conversion->{'factor'} ) ) {
 
@@ -113,10 +122,9 @@ sub convert {
           if float_is_infinite( $conversion->{'factor'} )
           || float_is_nan( $conversion->{'factor'} );
 
+        # if factor is < 0 and first thing can't be negative ...
         return
-          if $conversion->{'factor'} < 0
-          && !($matches->{'can_be_negative_1'}
-            && $matches->{'can_be_negative_2'} );
+          if $conversion->{'factor'} < 0 && !$matches->{'can_be_negative_1'};
     }
     else {
      # if it doesn't look like a number, and it contains a number (e.g., '6^2'):
@@ -130,18 +138,86 @@ sub convert {
 
     # run the conversion:
     # temperatures don't have 1:1 conversions, so they get special treatment:
+    my $factor;
     if ( $matches->{'type_1'} eq 'temperature' ) {
-        $matches->{'result'} = convert_temperatures( $matches->{'from_unit'},
-            $matches->{'to_unit'}, $conversion->{'factor'} );
+        $factor = convert_temperatures(
+            {
+                from_unit => $matches->{'from_unit'},
+                to_unit   => $matches->{'to_unit'},
+                factor    => $conversion->{'factor'},
+            }
+        );
     }
     else {
-        $matches->{'result'} = $conversion->{'factor'} *
+        $factor = $conversion->{'factor'} *
           ( $matches->{'factor_2'} / $matches->{'factor_1'} );
     }
 
+    return if $factor < 0 && !( $matches->{'can_be_negative_2'} );
+
+    $matches->{'result'} = precision(
+        {
+            factor    => $factor,
+            precision => $conversion->{'precision'} || $DEFAULT_PRECISION,
+        }
+    );
+
     return $matches;
 }
 
+# thanks, @mwmiller!
+sub parse_number {
+    my $in = shift;
+
+    my $out =
+      ( $in =~ /^(-?\d*(?:\.?\d+))\^(-?\d*(?:\.?\d+))$/ ) ? $1**$2 : $in;
+
+    return $in if $out =~ /[^\d]/;    # elo
+
+    return 0 + $out;
+}
+
+sub _build_types {
+    my $self = shift;
+
+    return get_units( $self->{data_file} );
+}
+
+sub get_units {
+    my $self = shift;
+
+    #my $data_file = shift // 'NO_DATA_FILE';
+    my $data_file = defined(shift) ? shift : 'NO_DATA_FILE';
+
+    my $units;
+
+    if ( $data_file eq 'NO_DATA_FILE' ) {
+
+        $units = encode_json( old_get_units() );
+
+    }
+    else {
+        $units = read_file($data_file);
+    }
+
+    return decode_json($units);
+}
+
+# does it make sense to return X if precision is > 1?
+#   e.g., result = '0.999' with precision = '3' returns '1' atm
+#   e.g., result = '1.000' with precision = '3' returns '1' atm
+sub precision {
+    my $precision = shift;
+
+    if ( $precision->{'factor'} == 0 ) { return 0; }
+
+    my $f = $precision->{'factor'} * ( 10**$precision->{'precision'} );
+
+    my $r = int( $f + $f / abs( $f * 2 ) );
+
+    return $r / ( 10**$precision->{'precision'} );
+}
+
 sub old_get_units {
 
     # metric ton is base unit for mass
@@ -1188,7 +1264,7 @@ sub old_get_units {
         },
         {
             'unit'    => 'us gallon',
-            'factor'  => 1 / 3.785411784,
+            'factor'  => 1 / 3.78541178,
             'aliases' => [
                 'fluid gallon',
                 'us fluid gallon',
@@ -1259,42 +1335,6 @@ sub old_get_units {
     return \@types;
 }
 
-# thanks, @mwmiller!
-sub parse_number {
-    my $in = shift;
-
-    my $out =
-      ( $in =~ /^(-?\d*(?:\.?\d+))\^(-?\d*(?:\.?\d+))$/ ) ? $1**$2 : $in;
-
-    return $in if $out =~ /[^\d]/;    # elo
-
-    return 0 + $out;
-}
-
-sub _build_types {
-    my $self = shift;
-
-    return get_units( $self->{data_file} );
-}
-
-sub get_units {
-    my $self = shift;
-    my $data_file = shift // 'NO_DATA_FILE';
-
-    if ( $data_file eq 'NO_DATA_FILE' ) {
-        return old_get_units();
-    }
-
-    my $units;
-
-    open my $data_fh, '<', $data_file or die "Could not open $data_file: [$!]";
-    local $/ = undef;
-    $units = <$data_fh>;
-    close($data_fh);
-
-    return decode_json($units);
-}
-
 1;
 
 __END__
@@ -1304,7 +1344,9 @@ Convert::Pluggable - convert between various units of measurement
 
 =head1 VERSION
 
-Version 0.031
+Version 0.032
+
+Authoritative version is always here: https://github.com/elohmrow/Convert-Pluggable
 
 =head1 SYNOPSIS
 
@@ -1314,11 +1356,11 @@ convert between various units of measurement
 
 You my use this module standalone prior to v0.031; there the units are all hard-coded into the module:
 
-C<use Convert::Pluggable;>
+        use Convert::Pluggable;
 
-C<my $c = new Convert::Pluggable();>
+        my $c = new Convert::Pluggable()
 
-C<my $result = $c->convert( { 'factor' => '5', 'from_unit' => 'feet', 'to_unit' => 'inches', 'precision' => '3', } );>
+        my $c->convert( { 'factor' => '5', 'from_unit' => 'feet', 'to_unit' => 'inches', 'precision' => '3', });
 
 Starting in v0.031 you may still use this module stand-alone, but you may optionally provide as a constructor argument 
 the name of a data file (currently only supports JSON) - then you will be able to use C::P as a service.  See examples/service.pl 
@@ -1328,6 +1370,25 @@ See t/Convert-Pluggable.t for many more example uses.
 
 See https://ddh5.duckduckgo.com/?q=10000+minutes+in+microseconds for examples of test uses
 
+See https://github.com/elohmrow/Convert-Pluggable for the most recent version
+
+=head1 BUILDING
+
+If you don't find me on CPAN, do the following:
+    
+=over 4
+
+=item *
+
+git clone https://github.com/elohmrow/Convert-Pluggable
+cd Convert-Pluggable
+perl Makefile.PL
+make
+make test
+make install
+
+=back
+
 =head1 EXPORT
 
 convert
@@ -1434,16 +1495,14 @@ Special thanks to @mintsoft and @jagtalon
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2014-2016 Bradley Andersen. This program is free software; you can redistribute it and/or modify it under the
+Copyright (c) 2014-2016 Bradley Andersen. This program is free software; you may redistribute and/or modify it under the
 same terms as Perl itself.
 
-=head1 PRIOR ART
+=head1 PRIOR ART - but not really!
 
-Bot::BasicBot::Pluggable::Module::Convert
+Bot::BasicBot::Pluggable::Module::Convert relies on Math::Units
 
-relies on 
-
-Math::Units
+Convert::Temperature
 
 =head1 TODO
 
@@ -1524,6 +1583,18 @@ https://github.com/duckduckgo/zeroclickinfo-goodies/issues/1767
 
 =item *
 
+Convert to p6
+
+=item *
+
+support for various number formats?
+
+=item *
+
+bignum support?
+
+=item *
+
 while massaging output is left to the implementation, there are some cases
 where answers might seem nonsensical, based on the input precision.  
 for example, converting '10mg to tons' with a precision of '3' gives '10 milligrams is 0.000 tons'
